/*
  test_nosync_dbe.c

  Last Update: Sat Oct 12 21:12:02 CEST 2002

  by Stefan Kral
  License: GPL Version 2 (or higher)
 */

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xdbe.h>

int main(int argc, char **argv)
{
  if (argc < 4) {
    printf("usage: test_dbe <Width> <Height> <NumberOfFlips>\n");
    return 127;
  } else {
    Display *dpy;
    int width = atoi(argv[1]), height = atoi(argv[2]), iter = atoi(argv[3]);

    if ((dpy = XOpenDisplay(NULL)) == NULL) {
      fprintf(stderr, "cannot open display\n");
      return 1;
    } else {
      int dbe_verLo, dbe_verHi;

      if (XdbeQueryExtension(dpy, &dbe_verHi, &dbe_verLo) == 0) {
	fprintf(stderr, "'DOUBLE-BUFFER' not supported by this display'\n");
	return 2;
      } else {
	Window root = DefaultRootWindow(dpy);
	int screen = DefaultScreen(dpy);
	GC gc1, gc2;
	XGCValues values;
	int j;
	unsigned long black = BlackPixel(dpy, screen);
	unsigned long white = WhitePixel(dpy, screen);
	XdbeSwapInfo swap_info;
	Window w = XCreateSimpleWindow(dpy, root, 
				       10, 10, width, height, 
				       0, black, white);
	Drawable buf = XdbeAllocateBackBufferName(dpy, w, XdbeUndefined);

	fprintf(stderr, "Using DBE version %d.%d\n", dbe_verHi, dbe_verLo);

	values.foreground = BlackPixel(dpy, screen);
	values.background = WhitePixel(dpy, screen);
	gc1 = XCreateGC(dpy, buf, (GCForeground | GCBackground), &values);
	  
	values.foreground = WhitePixel(dpy, screen);
	values.background = BlackPixel(dpy, screen);
	gc2 = XCreateGC(dpy, buf, (GCForeground | GCBackground), &values);
	  
	swap_info.swap_window = w;
	swap_info.swap_action = XdbeUndefined;

	XMapRaised(dpy, w);

	for (j = 0; j < iter; j++) {
	  XFillRectangle(dpy, buf, gc2, 0, 0, width, height);
	  XdbeSwapBuffers(dpy, &swap_info, 1);
	  XSync(dpy, 1);

	  XFillRectangle(dpy, buf, gc1, 0, 0, width, height);
	  XdbeSwapBuffers(dpy, &swap_info, 1);
	  XSync(dpy, 1);
	}
	  
	XdbeDeallocateBackBufferName(dpy, buf);
	
	XDestroyWindow(dpy, w); 		// also unmaps the window
	XCloseDisplay(dpy);
	  
	return 0;
      }
    }
  }
}
